#include "device.h"

#include "core/config.h"
#include "core/upower.h"

#include <QIcon>
#include <QFileInfo>
#include <QSettings>

using namespace core;

Device::Device(QString dbuspath, QObject *parent) :
    QObject(parent),
    m_interface(DBUS_INTERFACE_UPOWER, dbuspath, QDBusConnection::systemBus()),
    m_low(false),
    m_full(false),
    m_online(interface().online()) {

    m_settings.beginGroup(name());

    connect(&m_interface, SIGNAL(Changed()), this, SIGNAL(changed()));
    connect(&m_interface, SIGNAL(Changed()), this, SLOT(stateChanged()));
}

Device::~Device() {
    m_settings.endGroup();
}

QString Device::name() const {
    return UPower::nameFromPath(m_interface.path());
}

void Device::stateChanged() {
    switch(interface().type()) {
        case Battery: {
            // Emit signal if percent is low
            if(interface().percentage() <= lowValue()) {
                if(!m_low && alertOnLow() && isPresent()) {
                    m_low = true;
                    emit low();
                }
            } else {
                m_low = false;
            }

            // Emit signal if battery full
            if(interface().state() == FullyCharged) {
                if(!m_full && alertOnFull() && isPresent()) {
                    m_full = true;
                    emit full();
                }
            } else {
                m_full = false;
            }
        } break;
        case LinePower: {
            Alert a = alert();
            bool is_online = interface().online();

            if(is_online && !m_online) {
                if(a == Plugged || a == Plugged_Unplugged) {
                    emit online();
                }
                m_online = interface().online();
            } if(!is_online && m_online) {
                if(a == Unplugged || a == Plugged_Unplugged) {
                    emit offline();
                }
                m_online = interface().online();
            }
        } break;
    }
}

double Device::lowValue() const {
    return m_settings.value("LowValue", 15.00).toDouble();
}

void Device::setLowValue(double v) {
    m_settings.setValue("LowValue", v);
}

bool Device::alertOnLow() const {
    return m_settings.value("AlertOnLow", false).toBool();
}

void Device::setEnableLowAlert(bool b) {
    m_settings.setValue("AlertOnLow", b);
}

bool Device::alertOnFull() const {
    return m_settings.value("AlertOnFull", false).toBool();
}

void Device::setEnableFullAlert(bool b) {
    m_settings.setValue("AlertOnFull", b);
}

Device::Alert Device::alert() const {
    QSettings s;
    s.beginGroup(name());
    QString al = s.value("Alert", "Never").toString();
    s.endGroup();
    return stringToAlert(al);
}

void Device::setAlert(Device::Alert a) {
    if(a == alert())
        return;
    QSettings s;
    QString str;
    switch(a) {
        case Plugged:
            str = "Plugged";
            break;
        case Unplugged:
            str = "Unplugged";
            break;
        case Plugged_Unplugged:
            str = "Plugged & Unplugged";
            break;
        case Never:
        default:
            str = "Never";
            break;
    }
    s.beginGroup(name());
    s.setValue("Alert", str);
    s.endGroup();
    emit alertChanged(a);
}

Device::Alert Device::stringToAlert(const QString &alert) {
    if(alert == "Never") {
        return Never;
    } else if(alert == "Plugged") {
        return Plugged;
    } else if(alert == "Unplugged") {
        return Unplugged;
    } else if(alert == "Plugged & Unplugged") {
        return Plugged_Unplugged;
    }
    return Never;
}

void Device::refresh() {
    m_interface.Refresh();
}

QIcon Device::icon() const {
    QIcon icon;
    switch(interface().type()) {
        case LinePower:
            if(interface().online()) {
                icon = QIcon::fromTheme("powersupply", QIcon(":/battery/power-online"));
            } else {
                icon = QIcon::fromTheme("powersupply", QIcon(":/battery/power-offline"));
            }
            break;
        default:
        case Battery:
            if(isPresent()) {
                switch(interface().state()) {
                    default:
                    case Unknown:
                        icon = QIcon::fromTheme("battery-missing", QIcon(":/battery/nobattery"));
                        break;
                    case PendingCharge:
                    case Charging: {
                        double p = interface().percentage();
                        if(p >= 80) {
                            icon = QIcon::fromTheme("battery-charging-080", QIcon(":/battery/charging_80"));
                        } else if(p >= 60) {
                            icon = QIcon::fromTheme("battery-charging-060", QIcon(":/battery/charging_60"));
                        } else if(p >= 40) {
                            icon = QIcon::fromTheme("battery-charging-040", QIcon(":/battery/charging_40"));
                        } else if(p >= 20) {
                            icon = QIcon::fromTheme("battery-charging-caution", QIcon(":/battery/charging_caution"));
                        } else {
                            icon = QIcon::fromTheme("battery-charging-low", QIcon(":/battery/charging_low"));
                        }
                    }    break;
                    case PendingDischarge:
                    case Discharging: {
                        double p = interface().percentage();
                        if(p >= 99) {
                            icon = QIcon::fromTheme("battery-080", QIcon(":/battery/discharging_full"));
                        } else if(p >= 80) {
                            icon = QIcon::fromTheme("battery-080", QIcon(":/battery/discharging_80"));
                        } else if(p >= 60) {
                            icon = QIcon::fromTheme("battery-060", QIcon(":/battery/discharging_60"));
                        } else if(p >= 40) {
                            icon = QIcon::fromTheme("battery-040", QIcon(":/battery/discharging_40"));
                        } else if(p >= 20) {
                            icon = QIcon::fromTheme("battery-caution", QIcon(":/battery/discharging_caution"));
                        } else {
                            icon = QIcon::fromTheme("battery-low", QIcon(":/battery/discharging_low"));
                        }
                    }    break;
                    case FullyCharged:
                        icon = QIcon::fromTheme("battery-charging", QIcon(":/battery/charging_full"));
                        break;
                    case Empty:
                        icon = QIcon::fromTheme("battery-caution", QIcon(":/battery/battery-caution"));
                        break;
                }
            } else {
                icon = QIcon::fromTheme("battery-missing", QIcon(":/battery/nobattery"));
            }
            break;
    }
    return icon;
}

QString Device::prettyName() const {
    QString name;
    if(isPowerSupply()) {
        name += "Power Supply";
    } else {
        name += interface().model();
    }

    name += " '" + UPower::nameFromPath(m_interface.path()) + "'";

    return name;
}

QTime Device::timeLeft() const {
    QTime t(0, 0);
    if(m_interface.type() == Battery) {
        if(m_interface.state() == Discharging || m_interface.state() == PendingDischarge) {
            t = t.addSecs(m_interface.timeToEmpty());
        } else if(m_interface.state() == Charging || m_interface.state() == PendingCharge) {
            t = t.addSecs(m_interface.timeToFull());
        }
    }
    return t;
}

bool Device::isPresent() const {
    QFile f(m_interface.nativePath()+"/charge_now");
    f.open(QFile::ReadOnly);
    if(!f.exists())
        return m_interface.isPresent();
    char c;
    int r;
    r = f.read(&c, 1);
    f.close();
    return m_interface.isPresent() && r == 1;
}

