#include "gui/devicecombobox.h"

#include "core/application.h"

#include "core/device.h"
#include "core/upower.h"

using namespace gui;

DeviceComboBox::DeviceComboBox(QWidget *parent) : QComboBox(parent),
    m_batteryOnly(false) {
    connect(this, SIGNAL(currentIndexChanged(int)), this, SLOT(indexChanged(int)));
    connect(app->upower(), SIGNAL(mainDeviceChanged(core::Device*)), this, SLOT(setMainDevice(core::Device*)));
    connect(app->upower(), SIGNAL(deviceAdded(core::Device*)), this, SLOT(addDevice(core::Device*)));
    connect(app->upower(), SIGNAL(deviceRemoved(QString)), this, SLOT(removeDevice(QString)));
}

bool DeviceComboBox::shouldBeDisplayed(core::Device *d) {
    switch(d->interface().type()) {
        case core::Device::Battery:
            return d->isPresent();
            break;
        default:
            return !m_batteryOnly;
            break;
    }
}

void DeviceComboBox::addDevice(core::Device *d) {
    if(shouldBeDisplayed(d)) {
        addItem(d->icon(), d->prettyName(), d->name());
        if(d == app->upower()->mainDevice()) {
            setMainDevice(d);
        }
    }
    connect(d, SIGNAL(changed()), this, SLOT(updateDevice()));
}

void DeviceComboBox::removeDevice(const QString &name) {
    removeItem(findData(name));
}

void DeviceComboBox::updateDevice() {
    core::Device *d = qobject_cast<core::Device*>(sender());

    int pos = findData(d->name());
    if(shouldBeDisplayed(d)) {
        // Device should be displayed
        if(pos >= 0) {  // Already present just update icon
            setItemIcon(pos, d->icon());
        } else { // Not present add to ComboBox
            addItem(d->icon(), d->prettyName(), d->name());
            if(d == app->upower()->mainDevice()) {
                setMainDevice(d);
            }
        }
    } else {
        // It should be displayed
        if(pos >= 0) {  // But it is currently displayed
                        // removing ...
            removeItem(pos);
        }
    }

    if(count() == 0) {
        setCurrentIndex(0);
        emit noDevices();
    }
}

void DeviceComboBox::setMainDevice(core::Device *d) {
    for(int i = 0; i < count(); ++i) {
        if(d && itemData(i).toString() == d->name()) {
           setItemData(i, true, Qt::UserRole + 1);
           setItemText(i, d->prettyName() + " (main)");
           setCurrentIndex(i);
        } else {
            bool isMainBattery = itemData(i, Qt::UserRole + 1).toBool();
            if(isMainBattery) {
                setItemText(i, app->upower()->device(itemData(i).toString())->prettyName());
            }
        }
    }

}

void DeviceComboBox::indexChanged(int idx) {
    QString name = itemData(idx).toString();
    if(app->upower()->deviceExists(name)) {
        core::Device *d = app->upower()->device(name);
        emit currentDeviceChanged(name);
        emit currentDeviceChanged(d);
    }
}

void DeviceComboBox::setOnlyBattery(bool b) {
    m_batteryOnly = b;
    clear();
    foreach(core::Device *d, app->upower()->devices().values()) {
        addDevice(d);
    }
    setMainDevice(app->upower()->mainDevice());
    if(count() == 0) {
        setCurrentIndex(0);
        emit noDevices();
    }
}

core::Device* DeviceComboBox::device(int idx) const {
    QString name = itemData(idx).toString();
    return app->upower()->device(name);
}

core::Device* DeviceComboBox::currentDevice() const {
    return device(currentIndex());
}
