#include "ucpufreqtablewidget.h"

#include "core/application.h"
#include "core/ucpufreq.h"
#include "core/ucpufreqdbus.h"
#include "core/cpudbus.h"
#include "core/dbuswatcher.h"

#include <QComboBox>
#include <QHeaderView>

using namespace gui;

UCpufreqTableWidget::UCpufreqTableWidget(QWidget *parent) :
    QTableWidget(parent)
{
    connect(app->cpufreq(), SIGNAL(cpuAdded(core::CpuDBus*)), this, SLOT(addCPU(core::CpuDBus*)));
    connect(app->cpufreq(), SIGNAL(disconnected()), this, SLOT(unavailable()));
    connect(app->cpufreq(), SIGNAL(connected()), this, SLOT(available()));


    verticalHeader()->setResizeMode(0, QHeaderView::ResizeToContents);
    verticalHeader()->setResizeMode(1, QHeaderView::ResizeToContents);
    verticalHeader()->setResizeMode(2, QHeaderView::ResizeToContents);

    if(app->cpufreq()->isConnected()) {
        available();
    }
}

void UCpufreqTableWidget::addCPU(core::CpuDBus *cpu) {
    // Add row
    int row = rowCount();
    insertRow(row);

    // Label Item
    QTableWidgetItem *label = new QTableWidgetItem(QString::number(row));
    label->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
    label->setTextAlignment(Qt::AlignCenter);

    // Governors
    QComboBox *governorCbx = new QComboBox;
    m_governorsCbx.insert(governorCbx, row);
    governorCbx->addItems(m_governors);

    // Frequencies
    QComboBox *frequencyCbx = new QComboBox;
    m_frequenciesCbx.insert(frequencyCbx, row);
    frequencyCbx->addItems(m_frequencies_human);

    setItem(row, 0, label);
    setCellWidget(row, 1, governorCbx);
    setCellWidget(row, 2, frequencyCbx);

    m_cpus.append(cpu);

    updateValues(row);

    connect(cpu, SIGNAL(Changed()), this, SLOT(updateCPU()));
}

void UCpufreqTableWidget::updateValues(int row) {
    int idx = row;
    core::CpuDBus *cpu = m_cpus[idx];

    QComboBox *f = frequenciesComboBoxAt(row);
    QComboBox *g = governorsComboBoxAt(row);
    disconnectComboBoxs(f, g);
    QString governor = cpu->governor();
    qulonglong frequency = cpu->frequency();
    idx = f->findText(frequencyToString(frequency));

    f->setCurrentIndex(idx);

    idx = g->findText(governor);
    if(idx < 0) {
        // In case the module have been loaded in between
        updateGovernorsList();
        idx = g->findText(governor);
    }
    g->setCurrentIndex(idx);
    connectComboBoxs(f, g);
}

void UCpufreqTableWidget::connectComboBoxs(QComboBox *f, QComboBox *g) {
    connect(f, SIGNAL(currentIndexChanged(int)), this, SLOT(changeFrequency(int)));
    connect(g, SIGNAL(currentIndexChanged(int)), this, SLOT(changeGovernor(int)));
}

void UCpufreqTableWidget::disconnectComboBoxs(QComboBox *f, QComboBox *g) {
    disconnect(f, SIGNAL(currentIndexChanged(int)), this, SLOT(changeFrequency(int)));
    disconnect(g, SIGNAL(currentIndexChanged(int)), this, SLOT(changeGovernor(int)));
}

void UCpufreqTableWidget::updateGovernorsList() {
    m_governors = app->cpufreq()->interface()->governors();
    foreach(QComboBox *c, m_governorsCbx.keys()) {
        disconnect(c, SIGNAL(currentIndexChanged(int)), this, SLOT(changeGovernor(int)));
        c->clear();
        c->addItems(m_governors);
        connect(c, SIGNAL(currentIndexChanged(int)), this, SLOT(changeGovernor(int)));
    }
}

void UCpufreqTableWidget::refresh() {
    for(int i = 0; i < rowCount(); ++i) {
        updateValues(i);
    }
}

void UCpufreqTableWidget::updateCPU() {
    core::CpuDBus *cpu = qobject_cast<core::CpuDBus*>(sender());
    if(!cpu)
        return;
    // TODO: Use a map instead
    int row = m_cpus.indexOf(cpu);
    updateValues(row);
}

void UCpufreqTableWidget::available() {
    m_governors = app->cpufreq()->interface()->governors();
    m_frequencies = app->cpufreq()->interface()->frequencies();

    // Convert qulonglong to Human readable string
    foreach(qulonglong f, m_frequencies) {
        m_frequencies_human.append(frequencyToString(f));
    }

    setEnabled(true);
}

void UCpufreqTableWidget::unavailable() {
    setEnabled(false);
    clearContents();
    setRowCount(0);
    m_governors.clear();
    m_frequencies_human.clear();
    m_frequencies.clear();
    m_cpus.clear();
    m_frequenciesCbx.clear();
    m_governorsCbx.clear();
}

void UCpufreqTableWidget::changeGovernor(int idx) {
    if(idx >= 0 && idx < m_governors.size()) {
        QComboBox *cbx = qobject_cast<QComboBox*>(sender());
        if(!cbx) {
            return;
        }
        int row = rowOfGovernorsCbx(cbx);
        core::CpuDBus *cpu = m_cpus[row];
        if(!cpu) {
            return;
        }
        cpu->setGovernor(m_governors[idx]);
        updateValues(row);
    }
}

void UCpufreqTableWidget::changeFrequency(int idx) {
    if(idx >= 0 && idx < m_frequencies.size()) {
        QComboBox *cbx = qobject_cast<QComboBox*>(sender());
        if(!cbx) {
            return;
        }
        int row = rowOfFrequenciesCbx(cbx);
        core::CpuDBus *cpu = m_cpus[row];
        if(!cpu) {
            return;
        }
        cpu->setFrequency(m_frequencies[idx]);
        updateValues(row);
    }
}

QString UCpufreqTableWidget::frequencyToString(qulonglong f) {
    if(f > ( 1000 * 1000)) {
        return QString("%1 Ghz").arg((double) f/1000/1000);
    } else if( f > 1000 ) {
        return QString("%1 Mhz").arg((double) f/1000);
    } else {
        return QString("%1").arg(f);
    }
}
